/*
 * Effects Library
 */

#include "effects_library.h"

/*
 * Creates a new effects library.
 */
EffectsLibrary *
effects_library_new ( void )
{
  EffectsLibrary *ret;

  ret = g_malloc ( sizeof(EffectsLibrary) );
  ret->table = g_hash_table_new_full ( g_str_hash, g_str_equal, NULL, NULL );

  effects_library_add ( ret, &EffectRingMod );
  effects_library_add ( ret, &EffectFlanger );
  effects_library_add ( ret, &EffectCompressor );

  return ret;
}


/*
 * Adds a new effect to the library.
 */
void
effects_library_add ( EffectsLibrary *el,
                      Effect *e )
{
  g_assert ( el != NULL );
  g_assert ( e != NULL );

  g_hash_table_insert ( el->table, e->name, e );
}


/*
 * Returns an effect copied from the library.
 * NULL if the effect is not found.
 */
Effect *
effects_library_get_new ( EffectsLibrary *el,
                          gchar *name )
{
  Effect *e;

  g_assert ( el != NULL );

  e = g_hash_table_lookup ( el->table, name );

  if ( e == NULL )
    return NULL;

  return e->clone ( e );
}


/*
 * Returns an effect of the library.
 * NULL if the effect is not found.
 */
Effect *
effects_library_get ( EffectsLibrary *el,
                      gchar *name )
{
  g_assert ( el != NULL );

  return g_hash_table_lookup ( el->table, name );
}


/*
 * Applies a function to every element of the library
 */
void
effects_library_foreach ( EffectsLibrary *el,
                          GHFunc func,
                          gpointer data )
{
  g_assert ( el != NULL );

  g_hash_table_foreach ( el->table, func, data );
}


/*
 * Removes an effect from the library.
 */
void
effects_library_remove ( EffectsLibrary *el,
                         gchar *name )
{
  g_assert ( el != NULL );

  g_hash_table_remove ( el->table, name );
}


/*
 * Destroys the effects library.
 */
void
effects_library_destroy ( EffectsLibrary *el )
{
  g_assert ( el != NULL );

  g_hash_table_destroy ( el->table );
}
