/*
 * Wave
 */

#ifndef __WAVE_H__
#define __WAVE_H__

#include <glib.h>

#include "wave_generators_library.h"
#include "effects_library.h"


typedef struct st_wave
{
  gchar *name;
  WaveGeneratorsLibrary *gen_lib;
  EffectsLibrary *effects_lib;

  GHashTable *generators;
  WaveGenerator *gen;
  GSList *effects;
  Effect *selected_effect;

  GtkVBox *vbox;
  GtkHBox *hbox_gen;
  GtkHBox *hbox_effects;
  GtkComboBox *combo_gen;
  GtkTreeView *tree_effects;

  GMutex *mutex;

} Wave ;


/*
 * Constructors
 */
Wave *
wave_new ( WaveGeneratorsLibrary *,
           EffectsLibrary * );

Wave *
wave_new_with_name ( WaveGeneratorsLibrary *,
                     EffectsLibrary *,
                     const gchar * );

/*
 * Setting the wave's name
 */
void
wave_set_name ( Wave *,
                const gchar * );

/*
 * Cloning a wave
 */
Wave *
wave_clone ( Wave * );

/*
 * Make the user interface
 */
void
wave_make_ui ( Wave *,
               GtkBox * );

/*
 * Show/Hide the UI of a Wave
 */
void
wave_show_ui ( Wave * );

void
wave_hide_ui ( Wave * );

/*
 * Iterator
 */
gdouble
wave_iter ( Wave * );

/*
 * Reset
 */
void
wave_reset ( Wave * );

/*
 * Destructor
 */
void
wave_destroy ( Wave * );

#endif
