/*
 * Square Wave Generator
 * Based on the simple wave generator
 */

#include <math.h>
#include "wave_generator.h"
#include "wave_generator_simple.h"

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

/*
 * Cloning operation.
 * Returns a brand new square wave generator.
 */
gpointer
wave_generator_square_clone ( gpointer data )
{
  return wave_generator_simple_clone_from_instance ( data, &WaveGeneratorSquare );
}


/*
 * Iterates the wave generator.
 * Returns the new value.
 */
gdouble
wave_generator_square_iter ( gpointer data )
{
  gdouble ret;
  WaveGeneratorSimpleData *wgsd;

  if ( data == NULL )
    return 0.0;

  wgsd = (WaveGeneratorSimpleData *) data;

  ret = wgsd->x - ( floor(wgsd->x/( 2.0*M_PI )) * ( 2.0*M_PI ) );

  if ( ret < M_PI )
    ret = -1.0;
  else
    ret = 1.0;

  wgsd->x += wgsd->x_inc;

  return ret;
}


/*
 * Sine Wave Generator
 */
WaveGenerator WaveGeneratorSquare = 
  {
    "square",
    NULL,
    wave_generator_simple_init,
    wave_generator_square_clone,
    wave_generator_simple_make_ui,
    wave_generator_simple_show_ui,
    wave_generator_simple_hide_ui,
    wave_generator_square_iter,
    wave_generator_simple_reset,
    wave_generator_simple_destroy
  } ;
