/*
 * Wave Generators Library
 */

#include "wave_generators_library.h"

/*
 * Creates a new wave generators library.
 */
WaveGeneratorsLibrary *
wave_generators_library_new ( void )
{
  WaveGeneratorsLibrary *ret;

  ret = g_malloc ( sizeof(WaveGeneratorsLibrary) );
  ret->table = g_hash_table_new_full ( g_str_hash, g_str_equal, NULL, NULL );

  wave_generators_library_add ( ret, &WaveGeneratorSine );
  wave_generators_library_add ( ret, &WaveGeneratorSquare );
  wave_generators_library_add ( ret, &WaveGeneratorTriangle );
  wave_generators_library_add ( ret, &WaveGeneratorSaw );

  return ret;
}


/*
 * Adds a new wave generator to the library.
 */
void
wave_generators_library_add ( WaveGeneratorsLibrary *wgl,
                              WaveGenerator *wg )
{
  g_assert ( wgl != NULL );
  g_assert ( wg != NULL );

  g_hash_table_insert ( wgl->table, wg->name, wg );
}


/*
 * Returns a wave generator copied from the library.
 * NULL if the wave generator is not found.
 */
WaveGenerator *
wave_generators_library_get_new ( WaveGeneratorsLibrary *wgl,
                                  gchar *name )
{
  WaveGenerator *wg;

  g_assert ( wgl != NULL );

  wg = g_hash_table_lookup ( wgl->table, name );

  if ( wg == NULL )
    return NULL;

  return wg->clone ( wg->data );
}


/*
 * Returns a wave generator of the library.
 * NULL if the wave generator is not found.
 */
WaveGenerator *
wave_generators_library_get ( WaveGeneratorsLibrary *wgl,
                              gchar *name )
{
  g_assert ( wgl != NULL );

  return g_hash_table_lookup ( wgl->table, name );
}


/*
 * Applies a function to every element of the library
 */
void
wave_generators_library_foreach ( WaveGeneratorsLibrary *wgl,
                                  GHFunc func,
                                  gpointer data )
{
  g_assert ( wgl != NULL );

  g_hash_table_foreach ( wgl->table, func, data );
}


/*
 * Removes a wave generator from the library.
 */
void
wave_generators_library_remove ( WaveGeneratorsLibrary *wgl,
                                 gchar *name )
{
  g_assert ( wgl != NULL );

  g_hash_table_remove ( wgl->table, name );
}


/*
 * Destroys the wave generators library.
 */
void
wave_generators_library_destroy ( WaveGeneratorsLibrary *wgl )
{
  g_assert ( wgl != NULL );

  g_hash_table_destroy ( wgl->table );
}
